-- ****************************************************************************
-- ****************************************************************************
--
-- Converting ratings to percentages
--
-- ****************************************************************************
-- ****************************************************************************

import "MaKoPlugins.Utils";

-- ****************************************************************************
-- ****************************************************************************
--
-- Rating to percentage conversions. Reference:
--
--     https://lotro-wiki.com/index.php/Rating_to_percentage_formula
--
-- ****************************************************************************
-- ****************************************************************************

-- ----------------------------------------------------------------------------
-- New segment conversion function  for U20.1.2: Because of this, not
-- all conversions work yet.
-- ----------------------------------------------------------------------------

local function r2p(R, Pmax, C, Rcap)
    return Pmax * (C+1)/(C+Rcap/R)
end
    
-- ----------------------------------------------------------------------------
-- Segments
-- ----------------------------------------------------------------------------

local Segment = class()

function Segment:Constructor(Lmax, Pcap, Pmin, Pmax, C, RcapF, RcapC)
	self.Lmax  = Lmax
    self.Pcap  = Pcap
    self.Pmin  = Pmin
	self.Pmax  = Pmax
	self.C     = C
	self.RcapF = RcapF
    self.RcapC = RcapC
end

function Segment:Rcap(L)
    return L*self.RcapF + self.RcapC
end

function Segment:p(R, L)
    local p = self.Pmin + r2p(R, self.Pmax, self.C, self:Rcap(L))
    if self.Pcap then p = math.min(self.Pcap, p) end
    return p
end

-- ----------------------------------------------------------------------------
-- Stat data tables
-- ----------------------------------------------------------------------------

local segments = {
	["LightArmor"]  = {
	    Segment(104, 40,  0, 40, 1.6, 12800/100, 0),
	    Segment(105, 40,  0, 40, 1.6, 2720/21, 0),
	    Segment(115, 40,  0, 40, 1.6, 34000/9, -3420400/9),
	},
	["MediumArmor"] = {
	    Segment(104, 50,  0, 50, 10.0/7, 14866/100, 0),
	    Segment(105, 50,  0, 50, 10.0/7, 1047.625/7, 0),
	    Segment(115, 50,  0, 50, 10.0/7, 13095.3125/3, -1317388.4375/3),
	},
	["HeavyArmor"]  = {
	    Segment(104, 60,  0, 60, 1.2, 16650/100, 0),
	    Segment(105, 60,  0, 60, 1.2, 11608/700, 0),
	    Segment(115, 60,  0, 60, 1.2, 14600/3, -1468760/3),
	},

	["BlockPE"] = {
	    Segment( 20, 13,  0, 13, 2.0, 11500/100,     0),
	    Segment( 50, 13,  0, 13, 2.0,  9000/100,   500),
	    Segment( 105, 13,  0, 13, 2.0,  20000/100,   -5000),
	    Segment(115, 13,  0, 13, 2.0, 40000.0/9, -4024000.0/9),
	},
	["BParryE"] = {
	    Segment( 20, 13,  0, 13, 2.0, 11500/100,     0),
	    Segment( 50, 13,  0, 13, 2.0,  9000/100,   500),
	    Segment( 105, 13,  0, 13, 2.0,  20000/100,   -5000),
	    Segment(115, 13,  0, 13, 2.0, 40000.0/9, -4024000.0/9),
	},
	["BPEvade"] = {
	    Segment( 20, 13,  0, 13, 2.0, 11500/100,     0),
	    Segment( 50, 13,  0, 13, 2.0,  9000/100,   500),
	    Segment( 105, 13,  0, 13, 2.0,  20000/100,   -5000),
	    Segment(115, 13,  0, 13, 2.0, 40000.0/9, -4024000.0/9),
	},
	["PartialBPE"] = {
	    Segment( 20, 15,  0, 15, 2.5, 11250/100,      0),
	    Segment( 50, 15,  0, 15, 2.5,  7500/100,    750),
	    Segment( 84, 17,  0, 17, 2.5, 77500/100, -34250),
	    Segment( 95, 20,  0, 20, 2.5, 77500/100, -34250),
	    Segment( 104, 35,  0, 35, 2.5, 77500/100, -34250),
	    Segment( 105, 35,  0, 35, 2.5, 9500/21, 0),
	    Segment(115, 35,  0, 35, 2.5, 118750/9, -11946250/9),
	},
	["PartialBPEMit"] = {
	    --[[
	    Segment( 50, 60, 10, 50, 50, 121210.0 /  34, 0),
	    Segment(nil, 60, 10, 50, 50, 287750.0 / 105, 0),
	    ]]--
	    Segment( 20, 50, 10, 50, 50, 3500.0,      0.0),
	    Segment( 49, 50, 10, 50, 50, 3650.0,  -3000.0),
	    Segment( 104, 50, 10, 50, 50, 1950.0,  82500.0),
	    Segment( 105, 50, 10, 50, 50, 57500/21,      0.0),
	    Segment(115, 50, 10, 50, 50, 718750/9,  -72306250/9),
	},

	["Resistance"] = {
	    Segment( 50, 30,  0, 30, 1.0,  18000/100, 0),
	    Segment( 105, 50,  0, 50, 1.0,  2600/7, 0),
	    Segment(115, 50,  0, 50, 1.0, 32500/3, 0-3269500/3),
	},

	["CritRate"] = {
	    Segment( 50, 15,  0, 15, 0.66,  7150/100,     0.0),
	    Segment( 84, 20,  0, 20, 1.00, 25000/100, -6900.0),
	    Segment( 104, 25,  0, 25, 1.00, 16500/100, 375.0),
	    Segment( 105, 25,  0, 25, 1.00, 3550/21, 0),
	    Segment(115, 25,  0, 25, 1.00, 44375/9,  -4464125/9),
	},
	["DevRate"] = {
	    Segment( 50, 10,  0, 10, 2.00, 16000/100,     0.0),
	    Segment( 104, 10,  0, 10, 2.00, 16600/100,  -300.0),
	    Segment( 105, 10,  0, 10, 2.00, 3440/21,  0),
	    Segment(115, 10,  0, 10, 2.00, 43000/9, -4325800/9),
	},
	["CritMag"] = { 
		Segment(105, 50, 0, 50, 1.00, 30000/100, 0.0),
		Segment(115, 50, 0, 50, 1.00, 8750, -880250),
	 },
	["CritDef"] = { 
		Segment(105,50, 0, 50, 1.00, 10000/100, 0.0),
		Segment(115, 50, 0, 50, 1.00, 8750/3, -880250/3),	},
	["Finesse"] = { 
		Segment(105, 50, 0, 50, 10.00, 4000, 0.0),
		Segment(115, 50, 0, 50, 10.00, 350000/3, -35210000/3), },

	["OutHeals"] = {
	    Segment( 20, 30,  0, 30, 0.43, 17150/100,      0.0),
	    Segment( 50, 50,  0, 50, 1.00, 40000/100,      0.0),
	    Segment( 105, 70,  0, 70, 1.40, 77700/100,      -10850),
	    Segment(115, 70,  0, 70, 1.40, 353675/18, -35579705/18),
	},

	["IncHeals"] = {
	    Segment( 50, 15,  0, 15, 1.00, 7200/100,  0.0),
	    Segment(104, 25,  0, 25, 1.00, 24300/100, -8550.0),
	    Segment(105, 25,  0, 25, 1.00, 3400/21,  0.0),
	    Segment(115, 25,  0, 25, 1.00, 42500/9, -4275500/9),
	},

	["Mastery"] = {
        Segment(20, 40, 0, 40, 1.0/9, 14445/100, 0),
        Segment(49, 40, 0, 40, 1.0/9, 7733/58, 6451/29),
        Segment(50, 40, 0, 40, 1.0/9, 12209, 0),
        Segment(59, 80, 0, 80, 1.0/9, 30550/100, -1691.5),
        Segment(60, 80, 0, 80, 1.0/9, 2500/9, 0),
        Segment(99, 200, 0, 200, 1.0/9, 62489/76, -560739/76),
        Segment(100, 200, 0, 200, 1.0/9, 2225/3, 0),
        Segment(104, 400, 0, 400, 1.0/9, 13300/300, 461567/3),
        Segment(105, 400, 0, 400, 1.0/9, 9900/7, 0),
        Segment(115, 400, 0, 400, 1.0/9, 4125, -251625),
    },
    
--[[
	["IncHeals"] = {
		Segment(15, 100.0, 1190/3.0),
		Segment(10, 100.0, 2380/3.0)
	},
--]]
}

--[[
local linears = {
    ["Mastery"] = {
        Linear( 20, function(L) return 14.6 end,        0,  40.0),
        Linear( 30, function(L) return 24.2-0.48*L end, 0,  40.0),
        Linear( 40, function(L) return 17-0.24*L end,   0,  40.0),
        Linear( 50, function(L) return 13.4-0.15*L end, 0,  40.0),
        Linear( 60, function(L) return 11.4-0.11*L end, 0, 200.0),
        Linear( 70, function(L) return 10.2-0.09*L end, 0, 200.0),
        Linear( 80, function(L) return 7.4-0.05*L end,  0, 200.0),
        Linear( 90, function(L) return 6.6-0.04*L end,  0, 200.0),
        Linear(100, function(L) return 5.7-0.03*L end,  0, 200.0),
        Linear(105, function(L) return 2.7 end,         0, 400.0),
    },
}
]]--

-- ----------------------------------------------------------------------------

local function findSegment(segments, L)
	if segments then
	    for _, segment in ipairs(segments) do
		    if segment.Lmax == nil or L <= segment.Lmax then
		        return segment
		    end
		end
	end
	return nil
end

function ratingToPercentage(key, R, L)
    if R ~= nil then
        local segment = findSegment(segments[key], L)
        if segment then return segment:p(R, L) end
    end
    return nil
end

function ratingCap(key, L)
    local segment = findSegment(segments[key], L)
    if segment and segment.Pcap then return segment:Rcap(L) end
    return nil
end

